from enum import Enum
import numpy as np

# Environment settings
NUM_ACTIONS = 11
NUM_MEASUREMENTS = 14
MAX_VEHICLES = 27
MAX_SPEED = 16.67
MAX_WAIT = 120  # Just an estimate of max waiting time to scale input
MIN_REW = 5

"""Parameters without command line input options"""

per = False
ddqn = False
"""DQN"""
PRETRAIN = False
FILENAME = 'traffic.net'
INITIAL_STATE = np.zeros((20, 5))
EXPLORE = 5
Q_LEARNING_RATE = 0.001
DISCOUNT = 0.8
UPDATE_RATE = 500
BATCH_SIZE = 32
REPLAY_CAPACITY = 2000000
# Prioritized experience replay parameters
ALPHA = 0.2
EPSILON = 0.001
# Importance sampling - beta should reach 1.0 by convergence, increased by eta each transition recorded
BETA = 0.6
ETA = 0.00025

"""PG"""
POLY_INPUT = np.zeros((NUM_ACTIONS, NUM_MEASUREMENTS))
POLY_LEARNING_RATE = 0.1

"""CMA-ES"""
CMA_POP = 8
CMA_SIMS = 2
CMA_VARIANCE = 0.1

WL_EL = ['s', 'r', 'r', 'r', 'r', 's', 'r', 'r', 'r', 'G', 'G', 's', 'r', 'r', 'r', 'r', 's', 'r', 'r', 'r', 'G', 'G']
E_W = ['s', 'r', 'r', 'r', 'r', 'G', 'G', 'G', 'G', 'r', 'r', 's', 'r', 'r', 'r', 'r', 'G', 'G', 'G', 'G', 'r', 'r']
SL_NL = ['s', 'r', 'r', 'r', 'G', 's', 'r', 'r', 'r', 'r', 'r', 's', 'r', 'r', 'r', 'G', 's', 'r', 'r', 'r', 'r', 'r']
NNL_SSL = ['G', 'G', 'G', 'G', 'g', 's', 'r', 'r', 'r', 'r', 'r', 'G', 'G', 'G', 'G', 'g', 's', 'r', 'r', 'r', 'r', 'r']


class Phase(Enum):
    W_L = [1]
    E = [2]
    N_L = [3]
    S = [4]
    E_L = [5]
    W = [6]
    S_L = [7]
    N = [8]
    R = []
    WW_L = W + W_L
    EE_L = E + E_L
    NN_L = N + N_L
    SS_L = S + S_L


pre = 'laneAreaDetector.'
class LAD(Enum):
    __order__ = 'W_L E N_L S E_L W S_L N WW_L EE_L NN_L SS_L EW NS'     # Required to be compatible with python 2
    W_L = [pre + 'W3', pre + 'W4']
    E = [pre + 'E0', pre + 'E1', pre + 'E2']
    N_L = [pre + 'N4']
    S = [pre + 'S0', pre + 'S1', pre + 'S2', pre + 'S3']
    E_L = [pre + 'E3', pre + 'E4']
    W = [pre + 'W0', pre + 'W1', pre + 'W2']
    S_L = [pre + 'S4']
    N = [pre + 'N0', pre + 'N1', pre + 'N2', pre + 'N3']
    WW_L = W + W_L
    EE_L = E + E_L
    NN_L = N + N_L
    SS_L = S + S_L
    EW = E + W + E_L + W_L
    NS = N + S + N_L + S_L

ind_pre = 'inductionLoop.'
class IND(Enum):
    __order__ = 'W_L E N_L S E_L W S_L N WW_L EE_L NN_L SS_L EW NS'     # Required to be compatible with python 2
    W_L = [ind_pre + 'W3', ind_pre + 'W4']
    E = [ind_pre + 'E0', ind_pre + 'E1', ind_pre + 'E2']
    N_L = [ind_pre + 'N4']
    S = [ind_pre + 'S0', ind_pre + 'S1', ind_pre + 'S2', ind_pre + 'S3']
    E_L = [ind_pre + 'E3', ind_pre + 'E4']
    W = [ind_pre + 'W0', ind_pre + 'W1', ind_pre + 'W2']
    S_L = [ind_pre + 'S4']
    N = [ind_pre + 'N0', ind_pre + 'N1', ind_pre + 'N2', ind_pre + 'N3']
    WW_L = W + W_L
    EE_L = E + E_L
    NN_L = N + N_L
    SS_L = S + S_L
    EW = E + W + E_L + W_L
    NS = N + S + N_L + S_L


CMA_RESULT = [8.96263511e-01,5.92454721e-01,6.82241661e-01,1.25295517e+00
,5.74462484e-01,1.36851164e+00,7.29369807e-01,5.01492077e-01
,8.87187799e-01,9.21455309e-01,8.14761296e-01,1.21967011e+00
,5.49623603e-01,1.39640399e+00,1.96118201e+00,1.18300715e+00
,6.33746765e-01,1.16486831e+00,1.62973227e+00,1.31259024e+00
,1.27386795e+00,1.15424639e-01,1.61721198e+00,1.27673872e+00
,1.47089627e-01,1.34929469e+00,1.22278445e+00,1.45315516e-01
,5.00404914e-01,1.57881724e+00,2.15235912e+00,1.18834834e+00
,1.68886351e+00,1.31804801e+00,2.85471177e-01,7.48927156e-01
,1.12166298e+00,5.89725402e-01,1.16293624e+00,1.23503125e+00
,4.95681773e-01,1.09510662e+00,1.49543045e+00,1.17906173e+00
,9.14600020e-01,6.68259909e-01,1.64340963e+00,9.04610746e-01
,1.44375412e+00,1.87175167e+00,8.48392714e-01,1.11182454e+00
,2.56904709e+00,9.71780479e-01,9.91556987e-01,1.05246128e+00
,1.45117036e+00,7.08334596e-01,1.04590057e+00,1.31150687e+00
,1.71326113e-01,1.09556544e+00,1.10945623e+00,7.93295355e-01
,3.13481044e-04,1.32484854e-01,3.21535326e-01,1.48297283e+00
,6.53478568e-01,9.41701052e-01,7.05446356e-01,1.11994183e+00
,1.34634666e+00,1.42000807e+00,1.29486236e+00,4.29856116e-01
,1.04560457e+00,1.04727394e+00,1.16680453e+00,9.72114730e-01
,1.43969344e+00,1.21745600e+00,8.87026799e-01,6.60482883e-01
,1.66878266e+00,5.99674432e-01,1.45308027e+00,1.30747929e-01
,1.63079318e+00,1.99684355e+00,6.86290859e-01,1.53159479e-01
,7.62484662e-01,1.86234014e+00,6.46773876e-01,6.28815087e-01
,9.64660144e-01,3.23867293e-01,1.08584486e+00,5.85570580e-01
,1.13014662e+00,9.99210648e-01,1.30825782e+00,1.24551002e+00
,1.48161428e+00,7.89041940e-01,1.15824711e+00,9.75051142e-01
,9.77689747e-01,2.71929729e-01,1.10014152e+00,1.50055692e+00
,1.02339353e+00,3.93559751e-01,1.42568285e+00,7.48879883e-01
,1.16150830e+00,8.32504899e-01,1.60303913e+00,9.44292959e-01
,1.62103766e-01,1.27928301e+00,4.94026486e-01,1.63763000e+00
,7.56176542e-01,1.46264274e+00,1.31165917e+00,1.48900821e+00
,1.19936785e+00,1.04999174e+00,1.65616702e+00,8.60981104e-01
,1.22764968e+00,9.79317475e-01,4.13314614e-01,7.56439756e-01
,1.26314181e+00,6.55089651e-01,8.05946912e-01,1.57927438e+00
,6.91446901e-01,1.23261733e+00,1.51499865e+00,1.19328324e+00
,1.11819357e+00,9.57427832e-01,1.91366193e-01,8.58556446e-01
,1.07871321e+00,1.31146879e+00,1.16314029e+00,4.93464092e-01
,9.83780288e-01,1.16612343e+00,1.46361846e+00,1.42539492e+00
,1.91478875e-01,6.46146783e-01,1.23980714e+00,8.06406001e-01
,1.59628554e+00,1.28309884e+00,1.20742954e+00,1.72412223e+00
,2.05818765e+00,1.32212541e+00,8.71354448e-01,8.91342671e-01
,4.36452302e-02,1.18896030e+00,1.81976822e+00,7.61612221e-01
,9.89042249e-01,1.09734936e+00,1.91119282e+00,4.77249599e-01
,1.03166516e+00,2.33996781e-01,4.95718238e-01,8.45730362e-01
,1.12917546e+00,1.83480886e-01,7.65674162e-01,1.83698464e+00
,1.15245900e+00,1.78749101e+00,7.38176331e-01,1.14454493e+00
,1.03526379e+00,1.17639247e+00,6.88314928e-01,7.79832946e-01
,6.74289518e-01,1.17641644e+00,2.40099604e-01,1.54929171e+00
,1.33453259e+00,1.44366016e+00,7.75321774e-01,6.02091820e-01
,1.82458907e+00,7.99977092e-01,4.08856541e-02,2.12240093e-01
,1.04811443e+00,1.09757395e+00,4.88818475e-01,1.01308214e+00
,7.39019607e-01,3.02796113e-01,1.04993419e+00,1.14961493e+00
,6.45494994e-01,9.29840510e-01,1.27805995e+00,2.09144522e-01
,1.84830369e+00,5.15907178e-01,6.41151429e-01,8.52064709e-01
,9.49675327e-01,1.50908581e+00,1.84747353e+00,1.20620286e+00
,5.30037868e-01,1.55713619e+00,1.23297599e+00,1.03910183e+00
,3.82223357e-03,9.13423773e-01,4.22986261e-01,4.04508532e-01
,1.88418538e-01,1.91922339e+00,6.90615293e-01,1.63688652e+00
,1.04900524e+00,1.35547564e+00,5.84061412e-01,1.59807540e+00
,1.41935636e+00,2.12861158e+00,1.05090802e+00,6.78230784e-01
,1.08080634e-02,8.71017782e-01,9.76146663e-01,1.54191305e+00
,1.02428908e+00,1.21877858e+00,1.20129758e+00,1.16020158e+00
,1.35626034e+00,8.26132639e-01,1.81513852e+00,9.58797121e-01
,5.50954035e-01,4.80770903e-01,6.05980603e-02,1.05505595e+00
,1.23106219e+00,4.68042077e-01,6.68868875e-01,7.83590589e-01
,9.69117238e-01,1.14240090e+00,5.69776941e-01,9.06062376e-01
,2.55184380e-01,8.25423784e-01,1.44056856e+00,1.28758468e+00
,1.34946170e+00,1.54047705e+00,5.63384439e-02,3.85672430e-01
,5.02572960e-01,1.50058135e+00,1.31818874e+00,1.43549221e+00
,8.17507064e-01,1.25997799e+00,1.15716906e+00,7.86690135e-01
,1.74677637e+00,1.40857197e+00,1.05920162e+00,8.81869232e-01
,4.19828854e-01,7.98560797e-01,1.50171145e-02,1.65521723e+00
,4.94278735e-01,1.29419969e+00,1.26169645e+00,1.74837262e+00
,9.82880053e-01,1.63444524e+00,1.22612245e+00,1.49188447e+00
,1.88464184e+00,1.07095987e+00,1.66817581e+00,7.93177384e-01
,2.28802170e-01,1.39736733e+00,7.25809903e-01,7.87147146e-01
,1.18567634e+00,4.91326626e-01,1.12936351e+00,8.67316831e-01
,6.78790877e-01,7.08207639e-01,1.05873728e+00,1.07212718e+00
,1.22165293e+00,1.15296849e-01,1.33352095e+00,1.12101407e+00
,2.79582569e-01,1.22676654e+00,1.23082170e+00,1.25601399e+00
,1.34149124e+00,7.06194868e-01,1.70482392e-01,1.58550565e+00
,8.35064208e-01,1.64724478e+00,9.70613471e-01,6.33903575e-01
,1.28220517e+00,3.43329428e-01,1.28702598e+00,4.44165409e-01
,1.23154538e+00,6.99852136e-01,1.44588723e+00,6.60622779e-01
,1.01872666e+00,8.58366535e-01,9.87539163e-02,8.58808955e-01
,1.11637859e+00,1.52178206e+00,8.67770726e-01,9.71305827e-01
,1.19599622e+00,1.29497433e+00,1.23995617e+00,1.52953987e+00]
